interface HostInstance {
    url: string;
    accept: string;
    name: string;
}
interface CustomHost extends HostInstance {
}
interface User {
    token: string;
}
/**
 * Configuration for API request retries
 */
interface RetryConfig {
    /**
     * Whether to enable retry for failed requests
     * @default true
     */
    enabled?: boolean;
    /**
     * Maximum number of retry attempts
     * @default 3
     */
    maxRetries?: number;
    /**
     * Base delay between retries in milliseconds
     * Will be used with exponential backoff
     * @default 1000 (1 second)
     */
    baseDelay?: number;
    /**
     * Maximum delay between retries in milliseconds
     * @default 30000 (30 seconds)
     */
    maxDelay?: number;
    /**
     * HTTP status codes that should trigger a retry
     * By default, retries on rate limits (429) and server errors (500-599)
     * @default [429, 500, 501, 502, 503, 504, 507, 508, 509]
     */
    retryStatusCodes?: number[];
}
interface PositionCoords {
    x: number;
    y: number;
}
/**
 * Flexible image input types
 * This allows images to be passed in various formats for cross-platform compatibility
 */
type ImageInput = string | Blob | File | ArrayBuffer | Uint8Array | {
    data: Uint8Array;
} | {
    url: string;
} | HTMLImageElement | HTMLCanvasElement;
/**
 * Result from parsing an image
 */
interface ParsedImage {
    width: number;
    height: number;
    base64: string;
}
/**
 * Character caption for V4 prompts
 * Maps to char_caption and centers in the API
 */
interface CharacterCaption {
    charCaption: string;
    centers: PositionCoords[];
}
/**
 * Character prompt for V4.5 multi-character generation
 */
interface CharacterPrompt {
    prompt: string;
    uc: string;
    center: PositionCoords;
    enabled?: boolean;
}
/**
 * V4 caption format for prompts
 * Maps to base_caption and char_captions in the API
 */
interface V4CaptionFormat {
    baseCaption: string;
    charCaptions: CharacterCaption[];
}
/**
 * V4 prompt format with multi-character support
 * Maps to caption, use_coords and use_order in the API
 */
interface V4PromptFormat {
    caption: V4CaptionFormat;
    useCoords: boolean;
    useOrder: boolean;
}
/**
 * V4 format for negative prompts
 * Maps to caption and legacy_uc in the API
 */
interface V4NegativePromptFormat {
    caption: V4CaptionFormat;
    legacyUc: boolean;
}
interface Metadata {
    prompt: string;
    model?: Model;
    action?: Action;
    resPreset?: Resolution;
    negativePrompt?: string;
    qualityToggle?: boolean;
    ucPreset?: 0 | 1 | 2 | 3;
    width?: number;
    height?: number;
    nSamples?: number;
    steps?: number;
    scale?: number;
    dynamicThresholding?: boolean;
    seed?: number;
    extraNoiseSeed?: number;
    sampler?: Sampler;
    sm?: boolean;
    smDyn?: boolean;
    cfgRescale?: number;
    noiseSchedule?: Noise;
    image?: string;
    strength?: number;
    noise?: number;
    controlnetStrength?: number;
    controlnetCondition?: string;
    controlnetModel?: Controlnet;
    addOriginalImage?: boolean;
    mask?: string;
    referenceImageMultiple?: string[];
    referenceInformationExtractedMultiple?: number[];
    referenceStrengthMultiple?: number[];
    paramsVersion?: 1 | 2 | 3;
    autoSmea?: boolean;
    characterPrompts?: CharacterPrompt[];
    v4Prompt?: V4PromptFormat;
    v4NegativePrompt?: V4NegativePromptFormat;
    skipCfgAboveSigma?: number | null;
    useCoords?: boolean;
    legacyUc?: boolean;
    normalizeReferenceStrengthMultiple?: boolean;
    deliberateEulerAncestralBug?: boolean;
    preferBrownian?: boolean;
    legacy?: boolean;
    legacyV3Extend?: boolean;
}
interface ImageOptions {
    filename: string;
    data: Uint8Array;
    metadata?: Metadata;
}
interface DirectorRequestBase {
    req_type: string;
    width: number;
    height: number;
    image: string;
}
interface LineArtRequest extends DirectorRequestBase {
    req_type: "lineart";
}
interface SketchRequest extends DirectorRequestBase {
    req_type: "sketch";
}
interface BackgroundRemovalRequest extends DirectorRequestBase {
    req_type: "bg-removal";
}
interface DeclutterRequest extends DirectorRequestBase {
    req_type: "declutter";
}
interface ColorizeRequest extends DirectorRequestBase {
    req_type: "colorize";
    prompt: string;
    defry: number;
}
interface EmotionRequest extends DirectorRequestBase {
    req_type: "emotion";
    prompt: string;
    defry: number;
}
type DirectorRequest = LineArtRequest | SketchRequest | BackgroundRemovalRequest | DeclutterRequest | ColorizeRequest | EmotionRequest;
interface ChangeEmotionOptions {
    image: ImageInput;
    emotion: EmotionOptions | string;
    prompt?: string;
    emotionLevel?: EmotionLevel | number;
}
interface NovelAIOptions {
    token: string;
    timeout?: number;
    retry?: RetryConfig;
}
interface NovelAIResponse {
    statusCode: number;
    statusText: string;
    headers: Record<string, string>;
    data: ArrayBuffer | ReadableStream<Uint8Array> | null;
}
interface NovelAIError extends Error {
    status?: number;
    statusText?: string;
}

declare enum Host {
    API = "api",
    WEB = "web"
}
/**
 * Create a custom host configuration
 * @param url - The URL of the host
 * @param accept - The accept header for the host
 * @param name - A name for the host (used for image filenames)
 * @returns A host instance
 */
declare function createCustomHost(url: string, accept: string, name?: string): HostInstance;
declare enum Model {
    V3 = "nai-diffusion-3",
    V3INP = "nai-diffusion-3-inpainting",
    V4 = "nai-diffusion-4-full",
    V4_CUR = "nai-diffusion-4-curated-preview",
    V4_5_CUR = "nai-diffusion-4-5-curated",
    FURRY = "nai-diffusion-furry",
    FURRYINP = "furry-diffusion-inpainting"
}
declare enum Controlnet {
    PALETTESWAP = "hed",
    FORMLOCK = "midas",
    SCRIBBLER = "fake_scribble",
    BUILDINGCONTROL = "mlsd",
    LANDSCAPER = "uniformer"
}
declare enum Action {
    GENERATE = "generate",
    INPAINT = "infill",
    IMG2IMG = "img2img"
}
declare enum DirectorTools {
    LINEART = "lineart",
    SKETCH = "sketch",
    BACKGROUND_REMOVAL = "bg-removal",
    EMOTION = "emotion",
    DECLUTTER = "declutter",
    COLORIZE = "colorize"
}
declare enum EmotionOptions {
    NEUTRAL = "neutral",
    HAPPY = "happy",
    SAD = "sad",
    ANGRY = "angry",
    SCARED = "scared",
    SURPRISED = "surprised",
    TIRED = "tired",
    EXCITED = "excited",
    NERVOUS = "nervous",
    THINKING = "thinking",
    CONFUSED = "confused",
    SHY = "shy",
    DISGUSTED = "disgusted",
    SMUG = "smug",
    BORED = "bored",
    LAUGHING = "laughing",
    IRRITATED = "irritated",
    AROUSED = "aroused",
    EMBARRASSED = "embarrassed",
    WORRIED = "worried",
    LOVE = "love",
    DETERMINED = "determined",
    HURT = "hurt",
    PLAYFUL = "playful"
}
declare enum EmotionLevel {
    NORMAL = 0,
    SLIGHTLY_WEAK = 1,
    WEAK = 2,
    EVEN_WEAKER = 3,
    VERY_WEAK = 4,
    WEAKEST = 5
}
declare enum Resolution {
    SMALL_PORTRAIT = "small_portrait",
    SMALL_LANDSCAPE = "small_landscape",
    SMALL_SQUARE = "small_square",
    NORMAL_PORTRAIT = "normal_portrait",
    NORMAL_LANDSCAPE = "normal_landscape",
    NORMAL_SQUARE = "normal_square",
    LARGE_PORTRAIT = "large_portrait",
    LARGE_LANDSCAPE = "large_landscape",
    LARGE_SQUARE = "large_square",
    WALLPAPER_PORTRAIT = "wallpaper_portrait",
    WALLPAPER_LANDSCAPE = "wallpaper_landscape"
}
declare const RESOLUTION_DIMENSIONS: Record<Resolution, [number, number]>;
declare enum Sampler {
    EULER = "k_euler",
    EULER_ANC = "k_euler_ancestral",
    DPM2S_ANC = "k_dpmpp_2s_ancestral",
    DPM2M = "k_dpmpp_2m",
    DPMSDE = "k_dpmpp_sde",
    DDIM = "ddim_v3"
}
declare enum Noise {
    NATIVE = "native",
    KARRAS = "karras",
    EXPONENTIAL = "exponential",
    POLYEXPONENTIAL = "polyexponential"
}

/**
 * Represents an image generated by NovelAI
 */
declare class Image {
    /**
     * The filename of the image
     */
    readonly filename: string;
    /**
     * Raw binary image data
     */
    readonly data: Uint8Array;
    /**
     * Optional metadata used to generate the image
     */
    readonly metadata?: Metadata;
    /**
     * Create a new Image instance
     *
     * @param options - Configuration options for the image
     * @param options.filename - The filename of the image
     * @param options.data - Raw binary data of the image
     * @param options.metadata - Optional metadata used to generate the image
     */
    constructor(options: ImageOptions);
    /**
     * Get the size of the image in bytes
     */
    get size(): number;
    /**
     * Convert the image data to a base64 string
     *
     * @returns Base64 encoded image data
     */
    toBase64(): string;
    /**
     * Create a data URL for the image
     *
     * @returns Data URL for the image (suitable for browser use)
     */
    toDataURL(): string;
    /**
     * Save the image to disk (Node.js environment only)
     *
     * @param outputPath - Path to save the image to. If a directory is provided,
     *                     the image is saved with its original filename in that directory.
     * @returns Promise that resolves to the full path of the saved file
     */
    save(outputPath: string): Promise<string>;
    /**
     * Create a Blob from the image data (browser environment only)
     *
     * @returns Blob representing the image
     */
    toBlob(): Blob;
    /**
     * Create a File object from the image data (browser environment only)
     *
     * @returns File object representing the image
     */
    toFile(): File;
}

/**
 * NovelAI client for interacting with the NovelAI image generation API
 */
declare class NovelAI {
    private token;
    private timeout;
    private headers;
    private retryConfig?;
    /**
     * Create a new NovelAI client
     *
     * @param options - Client configuration options
     * @param options.token - NovelAI access token
     * @param options.timeout - Request timeout in milliseconds (default: 30000)
     * @param options.retry - Configuration for request retries (default: enabled with 3 retries)
     */
    constructor(options: NovelAIOptions);
    /**
     * Generate images using NovelAI's API
     *
     * @param metadata - Generation parameters
     * @param host - API host to use (can be Host enum or HostInstance, default: WEB)
     * @param verbose - Whether to log cost information (default: false)
     * @param isOpus - Whether the user has Opus subscription (for cost calculation, default: false)
     * @returns Promise resolving to an array of Image objects
     */
    generateImage(metadata: Metadata, host?: Host | HostInstance, verbose?: boolean, isOpus?: boolean): Promise<Image[]>;
    /**
     * Process and validate metadata before sending to API
     *
     * @param metadata - User-provided metadata
     * @returns Processed metadata object
     */
    private processMetadata;
    /**
     * Get width and height from a resolution preset
     *
     * @param preset - Resolution preset
     * @returns [width, height] tuple
     */
    private getResolutionDimensions;
    /**
     * Use a Director tool with the specified request
     *
     * @param request - Director tool request
     * @param host - Host to use for the request (can be Host enum or HostInstance, default: WEB)
     * @returns Promise resolving to an Image object
     */
    useDirectorTool(request: DirectorRequest, host?: Host | HostInstance): Promise<Image>;
    /**
     * Handle decompression of ZIP response from Director API
     *
     * @param compressedData - Compressed ZIP data as ArrayBuffer
     * @returns Promise resolving to decompressed Uint8Array image data
     * @private
     */
    private handleDirectorDecompression;
    /**
     * Convert an image to line art
     *
     * @param image - Image input (path, Blob, File, URL, etc.)
     * @param host - Host to use for the request (optional)
     * @returns Promise resolving to an Image object
     */
    lineArt(image: ImageInput, host?: Host | HostInstance): Promise<Image>;
    /**
     * Convert an image to sketch
     *
     * @param image - Image input (path, Blob, File, URL, etc.)
     * @param host - Host to use for the request (optional)
     * @returns Promise resolving to an Image object
     */
    sketch(image: ImageInput, host?: Host | HostInstance): Promise<Image>;
    /**
     * Remove the background from an image
     *
     * @param image - Image input (path, Blob, File, URL, etc.)
     * @param host - Host to use for the request (optional)
     * @returns Promise resolving to an Image object
     */
    backgroundRemoval(image: ImageInput, host?: Host | HostInstance): Promise<Image>;
    /**
     * Declutter an image (remove noise, distractions, etc.)
     *
     * @param image - Image input (path, Blob, File, URL, etc.)
     * @param host - Host to use for the request (optional)
     * @returns Promise resolving to an Image object
     */
    declutter(image: ImageInput, host?: Host | HostInstance): Promise<Image>;
    /**
     * Colorize a sketch or line art
     *
     * @param image - Image input (path, Blob, File, URL, etc.)
     * @param host - Host to use for the request (optional)
     * @param prompt - Additional prompt to add to the request
     * @param defry - Defry value (0-5, default: 0)
     * @returns Promise resolving to an Image object
     */
    colorize(image: ImageInput, host?: Host | HostInstance, prompt?: string, defry?: number): Promise<Image>;
    /**
     * Change the emotion of a character in an image
     *
     * @param options - Options for changing emotion
     * @param options.image - Image input (path, Blob, File, URL, etc.)
     * @param options.emotion - Target emotion to change to
     * @param options.prompt - Additional prompt to add to the request
     * @param options.emotionLevel - Level of emotion change (0-5, optional)
     * @param host - Host to use for the request (optional)
     * @returns Promise resolving to an Image object
     */
    changeEmotion(options: ChangeEmotionOptions, host?: Host | HostInstance): Promise<Image>;
}

/**
 * Parse an image from various input types and return width, height, and base64 data
 * Supports browser and Node.js environments with various input formats
 *
 * @param input - Various image input formats (path, Blob, File, ArrayBuffer, etc.)
 * @returns Promise resolving to a ParsedImage object with width, height, and base64 data
 */
declare function parseImage(input: ImageInput): Promise<ParsedImage>;
/**
 * Creates a timestamp-based filename
 * @param prefix - Optional prefix for the filename
 * @param extension - File extension (default: 'png')
 * @returns Formatted filename
 */
declare function createFilename(prefix?: string, extension?: string): string;
/**
 * Ensures a directory exists, creating it if necessary
 * @param dir - Directory path
 */
declare function ensureDirectoryExists(dir: string): void;
/**
 * Deduplicate tags in a comma-separated string
 * @param prompt - Prompt string with tags separated by commas
 * @returns Deduplicated prompt string
 */
declare function deduplicateTags(prompt: string): string;
/**
 * Prepares metadata for API request
 * @param metadata - Metadata object with parameters
 * @returns Formatted API request payload
 */
declare function prepareMetadataForApi(metadata: Metadata): any;
/**
 * Converts file size to human-readable format
 * @param bytes - Size in bytes
 * @returns Formatted size string
 */
declare function formatFileSize(bytes: number): string;
/**
 * Calculates the Anlas cost based on parameters
 * @param metadata - Metadata object with parameters
 * @param isOpus - Whether user has Opus subscription
 * @returns Estimated Anlas cost
 */
declare function calculateCost(metadata: Metadata, isOpus?: boolean): number;
/**
 * Convert a base64 string to a Uint8Array
 * @param base64 - Base64 encoded string
 * @returns Uint8Array of the data
 */
declare function base64ToUint8Array(base64: string): Uint8Array;
/**
 * Convert a Uint8Array to a base64 string
 * @param array - Uint8Array data
 * @returns Base64 encoded string
 */
declare function uint8ArrayToBase64(array: Uint8Array): string;
/**
 * Save binary data to a file (Node.js environment only)
 * @param data - Binary data as Uint8Array
 * @param filepath - File path to save to
 */
declare function saveBinaryFile(data: Uint8Array, filepath: string): void;
declare const DEFAULT_RETRY_CONFIG: Required<RetryConfig>;
/**
 * Execute a function with retry logic
 *
 * @param fn - Async function to execute with retry logic
 * @param retryConfig - Configuration for retry behavior
 * @returns Promise that resolves with the result of the function
 */
declare function withRetry<T>(fn: () => Promise<T>, retryConfig?: RetryConfig): Promise<T>;

export { Action, type ChangeEmotionOptions, type CharacterCaption, type CharacterPrompt, Controlnet, type CustomHost, DEFAULT_RETRY_CONFIG, type DirectorRequest, DirectorTools, EmotionLevel, EmotionOptions, Host, type HostInstance, Image, type ImageOptions, type Metadata, Model, Noise, NovelAI, type NovelAIError, type NovelAIOptions, type NovelAIResponse, type PositionCoords, RESOLUTION_DIMENSIONS, Resolution, type RetryConfig, Sampler, type User, type V4CaptionFormat, type V4NegativePromptFormat, type V4PromptFormat, base64ToUint8Array, calculateCost, createCustomHost, createFilename, deduplicateTags, ensureDirectoryExists, formatFileSize, parseImage, prepareMetadataForApi, saveBinaryFile, uint8ArrayToBase64, withRetry };
